-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
local logTag = "bus"
M.type = "auxilliary"
M.relevantDevice = nil
M.defaultOrder = 1000

local htmlTexture = require("htmlTexture")
local destScreenName = nil
local destScreenCount = 1
local destScreenCountrear = 1
local destScreenTimer = 0
local eventDataNew = nil
local eventDataOld = nil
local destHtmlPath = nil
local displaytextarray = {}
local displaytextcolourarray = {}
local displayimgarray = {}
local displaytextreararray = {}
local displaytextcolourreararray = {}
local colourSign = false

local function updateGFX(dt)
	if #(displaytextarray) > 1 or #(displaytextreararray) > 1 then
		destScreenTimer = destScreenTimer + dt
		if destScreenName and destScreenTimer >= tonumber(eventDataOld.interval) then
		  --front and side signs
		  eventDataNew.displaytext = displaytextarray[destScreenCount]
		  eventDataNew.displaytextcolour = displaytextcolourarray[destScreenCount]
		  eventDataNew.displayimg = displayimgarray[destScreenCount]
		  destScreenCount = destScreenCount + 1
		  if destScreenCount > #(displaytextarray) then destScreenCount = 1 end
		  --rear sign
		  eventDataNew.displaytextrear = displaytextreararray[destScreenCountrear]
		  eventDataNew.displaytextcolourrear = displaytextcolourreararray[destScreenCountrear]
		  destScreenCountrear = destScreenCountrear + 1
		  if destScreenCountrear > #(displaytextreararray) then destScreenCountrear = 1 end
		  htmlTexture.call(destScreenName, "update", eventDataNew)
		  destScreenTimer = destScreenTimer % tonumber(eventDataOld.interval)
			--dump(destScreenCount)
			--dump(destScreenCountrear)
			--dump(#(displaytextarray))
			--dump(#(displaytextreararray))
			--dump(displaytextarray[1])
			--dump(displaytextarray[2])
			--dump(displaytextarray[3])
			--dump(displaytextreararray[1])
			--dump(displaytextreararray[2])
			--dump(displaytextreararray[3])
		end
    end
end

local function split (inputstr, sep)
        if inputstr == nil then inputstr = "#FFFFFF,#FFFFFF,#FFFFFF,#FFFFFF,#FFFFFF,#FFFFFF" end
        if sep == nil then sep = "," end
        local t={}
        for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
                table.insert(t, str)
        end
        return t
end

local function init(jbeamData)
  if jbeamData.destinationSignNode and v.data[jbeamData.destinationSignNode] then
    local destData = v.data[jbeamData.destinationSignNode]
    destScreenName = destData.destMaterialName or destScreenName
    destHtmlPath = destData.destHtmlPath
    local destWidth = destData.destTextureWidth or 256
    local destHeight = destData.destTextureHeight or 16
    local destText = destData.destText or ""
    local destRoute = destData.destRoute or ""
    local destExtra1 = destData.destExtra1 or ""
    local destExtra2 = destData.destExtra2 or ""
    local destExtra1r = destData.destExtra1r or ""
    local destExtra2r = destData.destExtra2r or ""
    local destInt = destData.destInt or 3
    local destImg = destData.destImg or 0
    local destExtra1Img = destData.destExtra1Img or 0
    local destExtra2Img = destData.destExtra2Img or 0
    local routeImg = destData.routeImg or 0
    local textColour = destData.textColour or "#FFFFFF,#FFFFFF,#FFFFFF,#FFFFFF,#FFFFFF,#FFFFFF"
    textColour = split(textColour, ",")
	colourSign = destData.colourSign or false

    if destHtmlPath and destScreenName then
      htmlTexture.create(destScreenName, destHtmlPath, destWidth, destHeight, 4, 'automatic')
		eventDataNew = {direction = destText, routeID = destRoute, displaytext = destText, displaytextrear = destRoute, extratext1 = destExtra1, extratext2 = destExtra2, interval = destInt, extratext1r = destExtra1r, extratext2r = destExtra2r, displayimg = destImg, destimg = destImg, extratext1img = destExtra1Img, extratext2img = destExtra2Img, routeimg = routeImg, coloursign = colourSign, textcolour = textColour, displaytextcolour = textColour[2], displaytextcolourrear = textColour[1], routetextcolour = textColour[1]}
		eventDataOld = eventDataNew
      htmlTexture.call(destScreenName, "update", eventDataOld )
    else
      log("E", logTag, "Got no html path or material name for destination, no HTML texture created!!!...")
    end
  end
end

local function onGameplayEvent(eventName, eventData)
  if eventName == "bus_onRouteChange" then
    if destScreenName then
		eventDataNew = eventData
		eventDataOld = eventData
		eventDataNew.coloursign = colourSign
		eventDataOld.coloursign = colourSign
		destScreenCount = 1
		destScreenCountrear = 1
		destScreenTimer = tonumber(eventDataOld.interval)
		displaytextarray = {}
		displaytextcolourarray = {}
		displayimgarray = {}
		displaytextreararray = {}
		displaytextcolourreararray = {}
		  table.insert(displaytextarray, eventDataOld.direction)
		  table.insert(displaytextcolourarray, split(eventDataOld.textcolour,",")[2])
		  if eventDataOld.extratext1 ~= "" then
			table.insert(displaytextarray, eventDataOld.extratext1)
			table.insert(displaytextcolourarray, split(eventDataOld.textcolour,",")[3])
		  end
		  if eventDataOld.extratext2 ~= "" then
			table.insert(displaytextarray, eventDataOld.extratext2)
			table.insert(displaytextcolourarray, split(eventDataOld.textcolour,",")[4])
		  end
		  table.insert(displayimgarray, eventDataOld.destimg)
		  if eventDataOld.extratext1 ~= "" then table.insert(displayimgarray, eventDataOld.extratext1img) end
		  if eventDataOld.extratext2 ~= "" then table.insert(displayimgarray, eventDataOld.extratext2img) end
		  if eventDataOld.routeID ~= "" then
			table.insert(displaytextreararray, eventDataOld.routeID)
			table.insert(displaytextcolourreararray, split(eventDataOld.textcolour,",")[1])
		  end
		  if eventDataOld.extratext1r ~= "" then
			table.insert(displaytextreararray, eventDataOld.extratext1r)
			table.insert(displaytextcolourreararray, split(eventDataOld.textcolour,",")[5])
		  end
		  if eventDataOld.extratext2r ~= "" then
			table.insert(displaytextreararray, eventDataOld.extratext2r)
			table.insert(displaytextcolourreararray, split(eventDataOld.textcolour,",")[6])
		  end
		  if #(displaytextreararray) == 0 then table.insert(displaytextreararray, "") end
		  eventDataNew.displaytext = displaytextarray[1]
		  eventDataNew.displaytextcolour = displaytextcolourarray[1]
		  eventDataNew.displayimg = displayimgarray[1]
		  eventDataNew.displaytextrear = displaytextreararray[1]
		  eventDataNew.displaytextcolourrear = displaytextcolourreararray[1]
		  eventDataNew.routetextcolour = displaytextcolourreararray[1]
		htmlTexture.call(destScreenName, "update", eventDataNew)
    end
    guihooks.trigger("BusDisplayUpdate", eventData)
    dump(eventData)
  end
end

M.init = init
M.updateGFX = updateGFX
M.onGameplayEvent = onGameplayEvent

return M
